<?php
/**
 * This file contains the base class for WPJM SPL gateways.
 *
 * @package wp-job-manager-simple-paid-listings
 */

// Exit if accessed directly.
if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

/**
 * WP_Job_Manager_SPL_Gateway class.
 */
abstract class WP_Job_Manager_SPL_Gateway {

	/**
	 * The settings of the gateway that are going to be displayed in admin.
	 *
	 * @var array
	 */
	protected $settings = array();

	/**
	 * Unique id for the gateway.
	 *
	 * @var string
	 */
	protected $gateway_id = '';

	/**
	 * Gateway name.
	 *
	 * @var string
	 */
	protected $gateway_name = '';

	/**
	 * The instances of all gateways. Each implementation has a single instance.
	 *
	 * @var array
	 */
	private static $instances = array();

	/**
	 * Return the instance of the called class.
	 *
	 * @return self
	 */
	public static function instance() {
		if ( ! isset( self::$instances[ static::class ] ) ) {
			self::$instances[ static::class ] = new static();
		}

		return self::$instances[ static::class ];
	}

	/**
	 * __construct function.
	 */
	public function __construct() {
		add_filter( 'wp_job_manager_spl_gateways', array( $this, 'add_gateway' ) );
		add_filter( 'wp_job_manager_spl_settings', array( $this, 'add_settings' ) );

		if ( ! in_array( static::class, array( 'WP_Job_Manager_SPL_PayPal', 'WP_Job_Manager_SPL_Stripe' ), true ) ) {
			// For backwards compatibility with custom payment gateways, add a filter for the preview handler.
			add_filter( 'wp_job_manager_spl_preview_complete', array( $this, 'legacy_preview_complete_handler' ), 10, 2 );
		}
	}

	/**
	 * Whether we should continue past preview step.
	 *
	 * @param bool $preview_complete Whether we should continue past the preview step (default: true).
	 * @param int  $job_id           Job ID for the job being submitted.
	 * @return bool
	 */
	public function legacy_preview_complete_handler( $preview_complete, $job_id ) {
		if ( ! $this->pay_for_listing( $job_id ) ) {
			return false;
		}

		return $preview_complete;
	}

	/**
	 * Add gateway to settings page.
	 *
	 * @access private
	 *
	 * @param array $gateways Existing gateways.
	 *
	 * @return array The filtered gateways.
	 */
	public function add_gateway( $gateways ) {
		$gateways[ $this->gateway_id ] = $this->gateway_name;

		return $gateways;
	}

	/**
	 * Add settings for the gateway
	 *
	 * @access private
	 *
	 * @param array $settings Existing settings.
	 *
	 * @return array The filtered settings.
	 */
	public function add_settings( $settings ) {
		return array_merge( $settings, $this->settings );
	}

	/**
	 * Initialize frontend hooks.
	 */
	public function init_frontend() {}

	/**
	 * Handle API calls (optional - used for IPN)
	 */
	public function api_handler() {}

	/**
	 * Handle the return page (optional - used for getting return values from gateways if posted)
	 */
	public function return_handler() {}

	/**
	 * Pay for a job listing action
	 *
	 * @param int $job_id The job id.
	 */
	public function pay_for_listing( $job_id ) {
		return false;
	}

	/**
	 * Payment is complete - update listing
	 *
	 * @param int $job_id Job ID.
	 */
	public function payment_complete( $job_id ) {
		$job = get_post( $job_id );

		if ( in_array( $job->post_status, array( 'pending_payment', 'expired' ), true ) ) {
			// Reset expiry.
			delete_post_meta( $job_id, '_job_expires' );

			// Update status.
			$update_job                = array();
			$update_job['ID']          = $job_id;
			$update_job['post_status'] = get_option( 'job_manager_submission_requires_approval' ) ? 'pending' : 'publish';

			if ( method_exists( 'WP_Job_Manager_Form_Submit_Job', 'apply_scheduled_date' ) ) {
				$job_schedule_listing_date = get_post_meta( $job_id, '_job_schedule_listing', true );
				WP_Job_Manager_Form_Submit_Job::apply_scheduled_date( $update_job, $job_schedule_listing_date );
			} else {
				$update_job['post_date']     = current_time( 'mysql' );
				$update_job['post_date_gmt'] = current_time( 'mysql', 1 );
			}

			wp_update_post( $update_job );
		}

		do_action( 'wp_job_manager_spl_payment_complete', $job_id );
	}

	/**
	 * Notify admin that payment is complete.
	 *
	 * @since 2.0.0
	 * @param  int  $job_id Job ID.
	 * @param  bool $is_listing_renewal Whether this is a listing renewal.
	 */
	public function payment_complete_notify_admin( $job_id, $is_listing_renewal ) {
		if ( $is_listing_renewal ) {
			// translators: %d is the job ID.
			$this->send_admin_email( $job_id, sprintf( __( 'Payment has been received in full for Job Listing #%d - this job has been automatically renewed.', 'wp-job-manager-simple-paid-listings' ), $job_id ) );
		} elseif ( get_option( 'job_manager_submission_requires_approval' ) ) {
			// translators: %d is the job ID.
			$this->send_admin_email( $job_id, sprintf( __( 'Payment has been received in full for Job Listing #%d - this job is ready for admin approval.', 'wp-job-manager-simple-paid-listings' ), $job_id ) );
		} else {
			// translators: %d is the job ID.
			$this->send_admin_email( $job_id, sprintf( __( 'Payment has been received in full for Job Listing #%d - this job has been automatically approved.', 'wp-job-manager-simple-paid-listings' ), $job_id ) );
		}
	}

	/**
	 * Send a message to admin about payment
	 *
	 * @param  int    $job_id The job id.
	 * @param  string $message The message to sent.
	 */
	public function send_admin_email( $job_id, $message ) {
		$message = "Hi,\n\n" . $message . "\n\nView this job: " . admin_url( 'post.php?post=' . $job_id . '&action=edit' );

		// translators: Placeholder %d is the job id.
		wp_mail( apply_filters( 'wp_job_manager_spl_admin_email', get_option( 'admin_email' ) ), sprintf( __( 'Job #%d Payment Update', 'wp-job-manager-simple-paid-listings' ), $job_id ), $message );
	}

	/**
	 * Gets the email address for the author of a job listing.
	 *
	 * @param int $job_id The job id.
	 *
	 * @return bool|string False if not found, otherwise returns email address.
	 */
	public function get_job_listing_author_email( $job_id ) {
		$job = get_post( $job_id );

		if ( empty( $job ) ) {
			return false;
		}

		$author = get_user_by( 'id', $job->post_author );

		return empty( $author->user_email ) ? false : $author->user_email;
	}
}
