<?php
/**
 * This file contains the gateway implementation for PayPal
 *
 * @package wp-job-manager-simple-paid-listings
 */

// Exit if accessed directly.
if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

/**
 * WP_Job_Manager_SPL_PayPal
 */
class WP_Job_Manager_SPL_PayPal extends WP_Job_Manager_SPL_Gateway {

	/**
	 * PayPal live url.
	 *
	 * @var string
	 */
	private $liveurl = 'https://www.paypal.com/cgi-bin/webscr';

	/**
	 * PayPal test url.
	 *
	 * @var string
	 */
	private $testurl = 'https://www.sandbox.paypal.com/cgi-bin/webscr';

	/**
	 * __construct function.
	 */
	public function __construct() {
		$this->gateway_id   = 'paypal';
		$this->gateway_name = __( 'PayPal Standard', 'wp-job-manager-simple-paid-listings' );
		$this->settings     = array(
			array(
				'name'  => 'job_manager_spl_paypal_email',
				'std'   => '',
				'label' => __( 'PayPal Email', 'wp-job-manager-simple-paid-listings' ),
				'desc'  => __( 'Your PayPal email address.', 'wp-job-manager-simple-paid-listings' ),
				'type'  => 'input',
				'class' => 'gateway-settings gateway-settings-paypal',
			),
			array(
				'name'  => 'job_manager_spl_paypal_identity_token',
				'std'   => '',
				'label' => __( 'PayPal Identity Token', 'wp-job-manager-simple-paid-listings' ),
				'desc'  => __( 'Optionally enable "Payment Data Transfer" (Profile > Website Payment Preferences) and then copy your identity token here. This will allow payments to be verified without the need for PayPal IPN.', 'wp-job-manager-simple-paid-listings' ),
				'type'  => 'input',
				'class' => 'gateway-settings gateway-settings-paypal',
			),
			array(
				'name'    => 'job_manager_spl_paypal_sandbox',
				'std'     => 'no',
				'label'   => __( 'PayPal Sandbox', 'wp-job-manager-simple-paid-listings' ),
				'desc'    => __( 'Enable PayPal Sandbox (used for testing)', 'wp-job-manager-simple-paid-listings' ),
				'options' => array(
					'yes' => __( 'Yes', 'wp-job-manager-simple-paid-listings' ),
					'no'  => __( 'No', 'wp-job-manager-simple-paid-listings' ),
				),
				'type'    => 'select',
				'class'   => 'gateway-settings gateway-settings-paypal',
			),
		);

		parent::__construct();
	}

	/**
	 * Initialize frontend hooks.
	 */
	public function init_frontend() {
		add_action( 'wp_job_manager_spl_handle_preview_step', array( $this, 'redirect_to_paypal' ) );
	}

	/**
	 * Redirect user to PayPal payment checkout.
	 *
	 * @param int $job_id Job ID for the job being submitted.
	 */
	public function redirect_to_paypal( $job_id ) {

		$expiry = get_post_meta( $job_id, '_job_expires', true );

		if ( isset( $_GET['action'] ) && 'renew' === $_GET['action'] && ! empty( $expiry ) ) { // phpcs:ignore WordPress.Security.NonceVerification.Recommended -- Input is used for comparison.
			update_post_meta( $job_id, '_renewal_status_' . $expiry, 'pending' );
		}

		$payment_link = $this->get_paypal_payment_link( $job_id );
		wp_redirect( $payment_link );

		exit;
	}

	/**
	 * Get PayPal payment link
	 *
	 * @param int $job_id The job id.
	 *
	 * @return string
	 */
	private function get_paypal_payment_link( $job_id ) {
		$step   = isset( $_REQUEST['step'] ) ? sanitize_text_field( wp_unslash( $_REQUEST['step'] ) ) : 0; // phpcs:ignore WordPress.Security.NonceVerification.Missing,  WordPress.Security.NonceVerification.Recommended -- Check happens later when possible. Input is used safely.
		$action = isset( $_REQUEST['action'] ) ? sanitize_text_field( wp_unslash( $_REQUEST['action'] ) ) : ''; // phpcs:ignore WordPress.Security.NonceVerification.Missing,  WordPress.Security.NonceVerification.Recommended -- Check happens later when possible. Input is used safely.
		$nonce  = isset( $_REQUEST['nonce'] ) ? sanitize_text_field( wp_unslash( $_REQUEST['nonce'] ) ) : ''; // phpcs:ignore WordPress.Security.NonceVerification.Missing,  WordPress.Security.NonceVerification.Recommended -- Check happens later when possible. Input is used safely.

		$invoice = strtoupper( str_replace( ' ', '-', get_bloginfo( 'name' ) ) ) . '-JOB-' . $job_id;

		if ( 'renew' === $action || 'relist' === $action ) {
			$invoice .= '-' . get_post_meta( $job_id, '_job_expires', true );
		}

		$paypal_args = apply_filters(
			'job_manager_spl_paypal_args',
			array(
				'cmd'           => '_cart',
				'business'      => get_option( 'job_manager_spl_paypal_email' ),
				'currency_code' => get_option( 'job_manager_spl_currency' ),
				'charset'       => 'UTF-8',
				'rm'            => 2,
				'upload'        => 1,
				'no_note'       => 1,
				'return'        => add_query_arg(
					array(
						'success' => 'true',
						'job_id'  => $job_id,
						'step'    => $step + 1,
						'action'  => $action,
						'nonce'   => $nonce,
					),
					get_permalink()
				),
				'cancel_return' => add_query_arg(
					array(
						'cancel' => 'true',
						'job_id' => $job_id,
						'step'   => $step,
						'action' => $action,
						'nonce'  => $nonce,
					),
					get_permalink()
				),
				'invoice'       => $invoice,
				'custom'        => $job_id,
				'notify_url'    => add_query_arg(
					array(
						'job-manager-api' => 'WP_Job_Manager_Simple_Paid_Listings',
						'gateway'         => $this->gateway_id,
						'action'          => $action,
					),
					home_url( '/' )
				),
				'no_shipping'   => 1,
				'item_name_1'   => __( 'New Job Listing', 'wp-job-manager-simple-paid-listings' ) . ' &quot;' . get_the_title( $job_id ) . '&quot;',
				'quantity_1'    => 1,
				'amount_1'      => WP_Job_Manager_Simple_Paid_Listings::get_job_listing_cost(),
			)
		);

		$paypal_args = http_build_query( $paypal_args, '', '&' );

		if ( get_option( 'job_manager_spl_paypal_sandbox' ) === 'yes' ) {
			$paypal_adr = $this->testurl . '?test_ipn=1&';
		} else {
			$paypal_adr = $this->liveurl . '?';
		}

		return $paypal_adr . $paypal_args;
	}

	/**
	 * API Handler
	 */
	public function api_handler() {
		if ( get_option( 'job_manager_spl_paypal_sandbox' ) === 'yes' ) {
			$paypal_adr = $this->testurl . '?test_ipn=1&';
			$host       = 'www.sandbox.paypal.com';
		} else {
			$paypal_adr = $this->liveurl . '?';
			$host       = 'www.paypal.com';
		}

		// Get recieved values from post data.
		$received_values  = array(
			'cmd' => '_notify-validate',
		);
		$received_values += stripslashes_deep( $_POST );

		// Send back post vars to paypal.
		$params = array(
			'body'        => $received_values,
			'sslverify'   => false,
			'timeout'     => 60,
			'user-agent'  => 'WP_Job_Manager',
			'httpversion' => '1.1',
			'headers'     => array(
				'host' => $host,
			),
		);

		// Post back to get a response.
		$response = wp_remote_post( $paypal_adr, $params );

		// check to see if the request was valid.
		if ( ! is_wp_error( $response ) && $response['response']['code'] >= 200 && $response['response']['code'] < 300 && ( strcmp( $response['body'], 'VERIFIED' ) === 0 ) ) {
			$this->valid_paypal_ipn_request();
		}
	}

	/**
	 * Return handler
	 *
	 * Alternative to IPN
	 */
	public function return_handler() {
		$posted = stripslashes_deep( $_REQUEST );

		if ( ! empty( $posted['custom'] ) ) {

			$job_id = absint( $posted['custom'] );
			$job    = get_post( $job_id );

			$posted['payment_status'] = strtolower( $posted['payment_status'] );
			$is_listing_renewal       = isset( $_GET['action'] ) && 'renew' === $_GET['action']; // phpcs:ignore WordPress.Security.NonceVerification.Recommended -- Input is used for comparison.
			$renewal_payment_verified = false;

			switch ( $posted['payment_status'] ) {
				case 'pending':
					if ( get_option( 'job_manager_spl_paypal_sandbox' ) !== 'yes' ) {
						// No pending payments for non-sandbox accounts.
						break;
					}
				case 'completed':
					// Only complete pending_payment jobs or listing renewals.
					if ( ! $is_listing_renewal && ! in_array( $job->post_status, array( 'pending_payment', 'expired' ), true ) ) {
						break;
					}

					// Validate Amount.
					if ( WP_Job_Manager_Simple_Paid_Listings::get_job_listing_cost() != $posted['mc_gross'] ) {
						$this->send_admin_email( $job_id, __( 'The PayPal amount received does not match the job listing fee - please manually check payment before approving this listing. The job has *not* been automatically approved.', 'wp-job-manager-simple-paid-listings' ) );

						break;
					}

					// Validate transaction.
					if ( get_option( 'job_manager_spl_paypal_sandbox' ) === 'yes' ) {
						$paypal_adr = $this->testurl;
						$host       = 'www.sandbox.paypal.com';
					} else {
						$paypal_adr = $this->liveurl;
						$host       = 'www.paypal.com';
					}

					$pdt = array(
						'body'        => array(
							'cmd' => '_notify-synch',
							'tx'  => $posted['txn_id'],
							'at'  => get_option( 'job_manager_spl_paypal_identity_token' ),
						),
						'sslverify'   => false,
						'timeout'     => 60,
						'user-agent'  => 'WP_Job_Manager',
						'httpversion' => '1.1',
						'headers'     => array(
							'host' => $host,
						),
					);

					// Post back to get a response.
					$response = wp_remote_post( $paypal_adr, $pdt );

					if ( is_wp_error( $response ) || false === strpos( $response['body'], 'SUCCESS' ) ) {
						break;
					}

					$this->mark_payment_verified( $job_id, $posted['txn_id'], $is_listing_renewal );
					$renewal_payment_verified = true;

					break;
			}

			if ( ! $renewal_payment_verified && $is_listing_renewal ) {
				add_filter( 'job_manager_job_submitted_content', [ $this, 'update_submission_messaging' ], 10, 2 );
			}
		}
	}

	/**
	 * Mark a PayPal payment as verified.
	 *
	 * @param int    $job_id The job id.
	 * @param string $txn_id The PayPal transaction id.
	 * @param bool   $is_listing_renewal Whether this is a renewal or not.
	 */
	private function mark_payment_verified( $job_id, $txn_id, $is_listing_renewal ): void {
		// Store PP Details
		update_post_meta( $job_id, 'Transaction ID', $txn_id );

		// Change the job status
		$this->payment_complete( $job_id );

		// Handle renewals.
		if ( $is_listing_renewal && class_exists( 'WP_Job_Manager_Helper_Renewals' ) ) {
			$expiry         = get_post_meta( $job_id, '_job_expires', true );
			$renewal_status = get_post_meta( $job_id, '_renewal_status_' . $expiry, true );

			if ( 'pending' === $renewal_status ) {
				WP_Job_Manager_Helper_Renewals::renew_job_listing( get_post( $job_id ) );
				update_post_meta( $job_id, '_renewal_status_' . $expiry, 'success' );
			}
		}

		// Notify admin.
		$this->payment_complete_notify_admin( $job_id, $is_listing_renewal );
	}

	/**
	 * Update the submission messaging for renewals that the payment is not verified.
	 *
	 * @access private
	 *
	 * @param string  $job_submitted_content The content to filter.
	 * @param WP_Post $job The job that was submitted.
	 */
	public function update_submission_messaging( $job_submitted_content, $job ) {
		global $wp_post_types;

		return '<div class="job-manager-message">' . wp_kses_post(
			sprintf(
			// translators: %1$s is the job listing post type name.
				__( '%1$s renewed successfully. The expiry date will be adjusted once payment is verified.', 'wp-job-manager' ),
				esc_html( $wp_post_types['job_listing']->labels->singular_name )
			)
		) . '</div>';
	}

	/**
	 * Triggered when a valid IPN request comes in
	 */
	public function valid_paypal_ipn_request() {
		$posted = stripslashes_deep( $_POST );

		if ( ! empty( $posted['custom'] ) ) {

			$job_id = absint( $posted['custom'] );
			$job    = get_post( $job_id );

			$posted['payment_status'] = strtolower( $posted['payment_status'] );

			if ( '1' === $posted['test_ipn'] && 'pending' === $posted['payment_status'] ) {
				$posted['payment_status'] = 'completed';
			}

			switch ( $posted['payment_status'] ) {
				case 'completed':
					$is_listing_renewal = isset( $_GET['action'] ) && 'renew' === $_GET['action']; // phpcs:ignore WordPress.Security.NonceVerification.Recommended -- Input is used for comparison.

					// Only complete pending_payment jobs or renewals.
					if ( ! in_array( $job->post_status, array( 'pending_payment', 'expired' ), true ) && ! $is_listing_renewal ) {
						return false;
					}

					// Validate Amount.
					if ( WP_Job_Manager_Simple_Paid_Listings::get_job_listing_cost() != $posted['mc_gross'] ) {
						$this->send_admin_email( $job_id, __( 'The PayPal amount recieved does not match the job listing fee - please manually check payment before approving this listing. The job has *not* been automatically approved.', 'wp-job-manager-simple-paid-listings' ) );

						return false;
					}

					// Store PP Details.
					update_post_meta( $job_id, 'Payer PayPal address', $posted['payer_email'] );
					$this->mark_payment_verified( $job_id, $posted['txn_id'], $is_listing_renewal );

					break;
				case 'pending':
					if ( get_option( 'job_manager_spl_paypal_sandbox' ) !== 'yes' ) {
						// No pending payments for non-sandbox accounts
						break;
					}

					$this->send_admin_email( $job_id, sprintf( __( 'PayPal payment is pending for Job Listing #%d - this job has *not* been automatically approved.', 'wp-job-manager-simple-paid-listings' ), $job_id ) );

					break;
				case 'reversed':
				case 'chargeback':
					$this->send_admin_email( $job_id, sprintf( __( 'The payment for Job Listing #%d was reversed. Please check this job/payment and change the status manually if need be.', 'wp-job-manager-simple-paid-listings' ), $job_id ) );

					break;
			}
		}

		exit;
	}
}

return WP_Job_Manager_SPL_PayPal::instance();
